@@ -1,6 +1,6 @@
 /* font-manager-codepoint-list.h
  *
- * Copyright (C) 2009 - 2021 Jerry Casiano
+ * Copyright (C) 2009-2022 Jerry Casiano
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,11 +21,14 @@
 #ifndef __FONT_MANAGER_CODEPOINT_LIST_H__
 #define __FONT_MANAGER_CODEPOINT_LIST_H__
 
+#include <hb.h>
 #include <glib.h>
 #include <glib-object.h>
 #include <json-glib/json-glib.h>
 
+#include "font-manager-regional-indicator-symbols.h"
 #include "unicode-codepoint-list.h"
+#include "unicode-info.h"
 
 G_BEGIN_DECLS
 
