@@ -1,6 +1,6 @@
 /* font-manager-freetype.c
  *
- * Copyright (C) 2009 - 2021 Jerry Casiano
+ * Copyright (C) 2009-2022 Jerry Casiano
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -252,7 +252,7 @@ font_manager_get_installation_target (GFile *font_file, GFile *target_dir,
 GType
 font_manager_fsType_get_type (void)
 {
-  static volatile gsize g_define_type_id__volatile = 0;
+  static gsize g_define_type_id__volatile = 0;
 
   if (g_once_init_enter (&g_define_type_id__volatile))
     {
@@ -278,7 +278,7 @@ font_manager_fsType_get_type (void)
 }
 
 /**
- * font_manager_fsType_to_string:
+ * font_manager_fsType_to_string: (skip)
  * @fstype: #FontManagerfsType
  *
  * Returns a description of the fsType field suitable for display.
