@@ -1,6 +1,6 @@
 /* font-manager-preview-pane.c
  *
- * Copyright (C) 2009 - 2021 Jerry Casiano
+ * Copyright (C) 2009-2022 Jerry Casiano
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -53,7 +53,7 @@ font_manager_preview_pane_page_to_string (FontManagerPreviewPanePage page)
 GType
 font_manager_preview_pane_page_get_type (void)
 {
-  static volatile gsize g_define_type_id__volatile = 0;
+  static gsize g_define_type_id__volatile = 0;
 
   if (g_once_init_enter (&g_define_type_id__volatile))
     {
@@ -77,6 +77,7 @@ struct _FontManagerPreviewPane
     GtkNotebook   parent_instance;
 
     gboolean                metadata_update_required;
+    gboolean                show_line_size;
     gdouble                 preview_size;
     gdouble                 glyph_preview_size;
     gchar                   *preview_text;
@@ -114,6 +115,7 @@ enum
     PROP_FONT,
     PROP_METADATA,
     PROP_ORTHOGRAPHY,
+    PROP_SHOW_LINE_SIZE,
     N_PROPERTIES
 };
 
@@ -169,6 +171,9 @@ font_manager_preview_pane_get_property (GObject *gobject,
         case PROP_SAMPLES:
             g_value_set_boxed(value, self->samples);
             break;
+        case PROP_SHOW_LINE_SIZE:
+            g_value_set_boolean(value, self->show_line_size);
+            break;
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, property_id, pspec);
     }
@@ -211,6 +216,9 @@ font_manager_preview_pane_set_property (GObject *gobject,
         case PROP_ORTHOGRAPHY:
             font_manager_preview_pane_set_orthography(self, g_value_get_object(value));
             break;
+        case PROP_SHOW_LINE_SIZE:
+            self->show_line_size = g_value_get_boolean(value);
+            break;
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, property_id, pspec);
     }
@@ -332,6 +340,18 @@ font_manager_preview_pane_class_init (FontManagerPreviewPaneClass *klass)
                                                             G_PARAM_STATIC_STRINGS |
                                                             G_PARAM_EXPLICIT_NOTIFY);
 
+    /**
+     * FontManagerFontPreviewPane:show-line-size:
+     *
+     * Whether to display line size in Waterfall preview or not.
+     */
+     obj_properties[PROP_SHOW_LINE_SIZE] = g_param_spec_boolean("show-line-size",
+                                                                NULL,
+                                                                "Whether to display Waterfall preview line size",
+                                                                TRUE,
+                                                                G_PARAM_STATIC_STRINGS |
+                                                                G_PARAM_READWRITE);
+
     g_object_class_install_properties(object_class, N_PROPERTIES, obj_properties);
     return;
 }
@@ -546,6 +566,7 @@ font_manager_preview_pane_init (FontManagerPreviewPane *self)
     self->properties = font_manager_properties_pane_new();
     self->license = font_manager_license_pane_new();
     self->metadata_update_required = TRUE;
+    self->show_line_size = TRUE;
     FontManagerFontPreviewMode mode = font_manager_font_preview_get_preview_mode(FONT_MANAGER_FONT_PREVIEW(self->preview));
     append_page(self, self->preview, font_manager_font_preview_mode_to_translatable_string(mode));
     append_page(self, self->character_map, _("Characters"));
@@ -570,6 +591,7 @@ font_manager_preview_pane_init (FontManagerPreviewPane *self)
     g_object_bind_property(self->preview, "preview-text", self, "preview-text", flags);
     g_object_bind_property(self->preview, "preview-mode", self, "preview-mode", flags);
     g_object_bind_property(self->preview, "samples", self, "samples", flags);
+    g_object_bind_property(self->preview, "show-line-size", self, "show-line-size", flags);
     g_object_bind_property(self->character_map, "preview-size", self, "character-map-preview-size", flags);
     g_signal_connect(self, "switch-page", G_CALLBACK(on_page_switch), NULL);
     return;
@@ -613,19 +635,14 @@ font_manager_preview_pane_show_uri (FontManagerPreviewPane *self, const gchar *u
         g_clear_error(&error);
         return;
     }
-    g_autoptr(JsonObject) orthography = font_manager_get_orthography_results(source);
-    if (!json_object_has_member(orthography, "Basic Latin")) {
-        GList *charset = font_manager_get_charset_from_filepath(path, index);
+    g_autofree gchar *sample = font_manager_get_sample_string(source);
+    if (sample) {
         if (!self->samples) {
             self->samples = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
             g_object_notify_by_pspec(G_OBJECT(self), obj_properties[PROP_SAMPLES]);
         }
-        g_autofree gchar *sample = font_manager_get_sample_string_for_orthography(orthography, charset);
-        if (sample) {
-            const gchar *description = json_object_get_string_member(source, "description");
-            g_hash_table_insert(self->samples, g_strdup(description), g_strdup(sample));
-        }
-        g_list_free(charset);
+        const gchar *description = json_object_get_string_member(source, "description");
+        g_hash_table_insert(self->samples, g_strdup(description), g_strdup(sample));
     }
     g_object_set(font, "source-object", source, NULL);
     font_manager_preview_pane_set_font(self, font);
@@ -696,15 +713,19 @@ font_manager_preview_pane_restore_state (FontManagerPreviewPane *self, GSettings
 }
 
 /**
- * font_manager_preview_pane_set_max_waterfall_size:
+ * font_manager_preview_pane_set_waterfall_size:
  * @self:           #FontManagerFontPreview
- * @size_points:    Maximum size to use for waterfall previews.
+ * @min_size:       Minimum point size to use for waterfall previews. (-1.0 to keep current)
+ * @max_size:       Maximum size to use for waterfall previews. (-1.0 to keep current)
+ * @ratio:          Waterfall point size common ratio. (-1.0 to keep current)
  */
 void
-font_manager_preview_pane_set_max_waterfall_size (FontManagerPreviewPane *self,
-                                                  gdouble size_points)
+font_manager_preview_pane_set_waterfall_size (FontManagerPreviewPane *self,
+                                              gdouble min_size,
+                                              gdouble max_size,
+                                              gdouble ratio)
 {
-    font_manager_font_preview_set_max_waterfall_size(FONT_MANAGER_FONT_PREVIEW(self->preview), size_points);
+    font_manager_font_preview_set_waterfall_size(FONT_MANAGER_FONT_PREVIEW(self->preview), min_size, max_size, ratio);
     return;
 }
 
