@@ -2,7 +2,7 @@
  *
  * Originally a part of Gucharmap
  *
- * Copyright (C) 2017 - 2021 Jerry Casiano
+ * Copyright (C) 2017-2022 Jerry Casiano
  *
  *
  * Copyright © 2004 Noah Levitt
@@ -123,7 +123,6 @@ enum
     STATUS_MESSAGE,
     MOVE_CURSOR,
     COPY_CLIPBOARD,
-    PASTE_CLIPBOARD,
     NUM_SIGNALS
 };
 
@@ -136,7 +135,7 @@ enum
     PROP_VADJUSTMENT,
     PROP_HSCROLL_POLICY,
     PROP_VSCROLL_POLICY,
-    PROP_ACTIVE_CHAR,
+    PROP_ACTIVE_CELL,
     PROP_CODEPOINT_LIST,
     PROP_FONT_DESC,
     PROP_PREVIEW_SIZE
@@ -262,12 +261,30 @@ unicode_character_map_y_offset (UnicodeCharacterMap *charmap, gint row)
     return y;
 }
 
+static gchar *
+get_text_for_cell (UnicodeCharacterMap *charmap, gint cell)
+{
+    GSList *codepoints = unicode_codepoint_list_get_codepoints(priv->codepoint_list, cell);
+    for (GSList *iter = codepoints; iter != NULL; iter = iter->next) {
+        gunichar wc = GPOINTER_TO_INT(iter->data);
+        if (wc > UNICODE_UNICHAR_MAX || !unicode_unichar_validate(wc))
+            return NULL;
+    }
+
+    gchar text [24];
+    gchar *p = text;
+    for (GSList *iter = codepoints; iter != NULL; iter = iter->next)
+        p += unicode_unichar_to_printable_utf8((gunichar) GPOINTER_TO_INT(iter->data), p);
+    p[0] = 0;
+    g_slist_free(codepoints);
+    return g_strdup(text);
+}
+
 static PangoLayout *
-layout_scaled_glyph (UnicodeCharacterMap *charmap, gunichar uc, double font_factor)
+layout_scaled_glyph (UnicodeCharacterMap *charmap, double font_factor)
 {
     PangoFontDescription *font_desc;
     PangoLayout *layout;
-    gchar buf[11];
 
     font_desc = pango_font_description_copy(priv->font_desc);
     double font_size = font_factor * pango_font_description_get_size(priv->font_desc);
@@ -279,22 +296,26 @@ layout_scaled_glyph (UnicodeCharacterMap *charmap, gunichar uc, double font_fact
 
     unicode_character_map_ensure_pango_layout(charmap);
     layout = pango_layout_new(pango_layout_get_context (priv->pango_layout));
+    PangoAttrList *attrs = pango_attr_list_new();
+    pango_attr_list_insert(attrs, pango_attr_fallback_new (FALSE));
+    pango_layout_set_attributes(layout, attrs);
+    pango_attr_list_unref(attrs);
     pango_layout_set_font_description(layout, font_desc);
-    buf[unicode_unichar_to_printable_utf8(uc, buf)] = '\0';
-    pango_layout_set_text(layout, buf, -1);
+    g_autofree gchar *glyph = get_text_for_cell(charmap, priv->active_cell);
+    pango_layout_set_text(layout, glyph, -1);
     pango_font_description_free(font_desc);
     return layout;
 }
 
 static cairo_surface_t *
-create_glyph_surface (UnicodeCharacterMap *charmap, gunichar wc, double font_factor)
+create_glyph_surface (UnicodeCharacterMap *charmap, double font_factor)
 {
     GtkWidget *widget = GTK_WIDGET(charmap);
     enum { PADDING = 16 };
     gint width, height;
     PangoRectangle char_rect;
 
-    PangoLayout *pango_layout = layout_scaled_glyph(charmap, wc, font_factor);
+    PangoLayout *pango_layout = layout_scaled_glyph(charmap, font_factor);
     pango_layout_get_pixel_extents(pango_layout, &char_rect, NULL);
     width  = char_rect.width + 2 * PADDING;
     height = char_rect.height + 2 * PADDING;
@@ -360,7 +381,6 @@ get_cell_at_xy (UnicodeCharacterMap *charmap, gint x, gint y)
     return cell;
 }
 
-
 static void
 draw_character (UnicodeCharacterMap *charmap,
                 cairo_t *cr,
@@ -369,20 +389,12 @@ draw_character (UnicodeCharacterMap *charmap,
                 gint col)
 {
     GtkWidget *widget = GTK_WIDGET(charmap);
-    int n, char_width, char_height;
+    int char_width, char_height;
     guint cell;
-    gchar buf[10];
 
     cell = get_cell_at_rowcol(charmap, row, col);
-    gunichar wc = unicode_codepoint_list_get_char(priv->codepoint_list, cell);
-
-    if (wc > UNICODE_UNICHAR_MAX ||
-        !unicode_unichar_validate(wc) ||
-        !unicode_unichar_isdefined(wc))
-        return;
-
-    n = unicode_unichar_to_printable_utf8(wc, buf);
-    pango_layout_set_text(priv->pango_layout, buf, n);
+    g_autofree gchar *text = get_text_for_cell(charmap, cell);
+    pango_layout_set_text(priv->pango_layout, text, -1);
 
     /* Keep the square empty if the font has no glyph for this cell. */
     if (pango_layout_get_unknown_glyphs_count(priv->pango_layout) > 0)
@@ -433,7 +445,9 @@ draw_square_bg (UnicodeCharacterMap *charmap,
 {
     GtkWidget *widget = GTK_WIDGET(charmap);
     gint cell = (gint) get_cell_at_rowcol(charmap, row, col);
-    gunichar wc = unicode_codepoint_list_get_char(priv->codepoint_list, cell);
+
+    GSList *codepoints = unicode_codepoint_list_get_codepoints(priv->codepoint_list, cell);
+    gunichar wc = (gunichar) GPOINTER_TO_INT(g_slist_nth_data(codepoints, 0));
 
     GtkStyleContext *ctx = gtk_widget_get_style_context(widget);
     gtk_style_context_save(ctx);
@@ -443,7 +457,7 @@ draw_square_bg (UnicodeCharacterMap *charmap,
         _state = GTK_STATE_FLAG_SELECTED;
     else if (cell == priv->active_cell)
         _state = GTK_STATE_FLAG_INSENSITIVE | GTK_STATE_FLAG_SELECTED;
-    else if (!wc || !unicode_unichar_validate(wc) || !unicode_unichar_isdefined(wc))
+    else if (!wc || !unicode_unichar_validate(wc))
         _state = GTK_STATE_FLAG_INSENSITIVE;
     else
         _state = GTK_STATE_FLAG_NORMAL;
@@ -483,45 +497,6 @@ draw_separators (UnicodeCharacterMap *charmap, cairo_t *cr)
     return;
 }
 
-static void
-unicode_character_map_set_active_cell (UnicodeCharacterMap *charmap, gint cell)
-{
-    GtkWidget *widget = GTK_WIDGET(charmap);
-
-    if (cell == priv->active_cell)
-        return;
-
-    if (cell < 0)
-        cell = 0;
-    else if (cell > priv->last_cell)
-        cell = priv->last_cell;
-
-    int old_active_cell = priv->active_cell;
-    int old_page_first_cell = priv->page_first_cell;
-
-    priv->active_cell = cell;
-
-    if (cell < priv->page_first_cell || cell >= priv->page_first_cell + priv->page_size) {
-        int old_row = old_active_cell / priv->cols;
-        int new_row = cell / priv->cols;
-        int new_page_first_cell = old_page_first_cell + ((new_row - old_row) * priv->cols);
-        int last_row = (priv->last_cell / priv->cols) + 1;
-        int last_page_first_row = last_row - priv->rows;
-        int last_page_first_cell = (last_page_first_row * priv->cols) + 1;
-        priv->page_first_cell = CLAMP(new_page_first_cell, 0, last_page_first_cell);
-        if (priv->vadjustment)
-            gtk_adjustment_set_value(priv->vadjustment, priv->page_first_cell / priv->cols);
-    } else if (gtk_widget_get_realized(widget)) {
-        /* Clear previous selection */
-        expose_cell(charmap, old_active_cell);
-        /* Update selected cell */
-        expose_cell(charmap, cell);
-    }
-
-    g_object_notify(G_OBJECT(charmap), "active-character");
-    return;
-}
-
 static void
 vadjustment_value_changed_cb (GtkAdjustment *vadjustment, UnicodeCharacterMap *charmap)
 {
@@ -555,7 +530,7 @@ unicode_character_map_set_font_desc_internal (UnicodeCharacterMap *charmap,
     unicode_character_map_set_active_cell(charmap, 1);
     update_scrollbar_adjustment(charmap);
     g_object_notify(G_OBJECT(charmap), "font-desc");
-    g_object_notify(G_OBJECT(charmap), "active-character");
+    g_object_notify(G_OBJECT(charmap), "active-cell");
     return;
 }
 
@@ -564,20 +539,20 @@ unicode_character_map_show_info(UnicodeCharacterMap *self, gdouble x, gdouble y)
 {
     g_return_if_fail(self != NULL);
 
-    if (priv->active_cell >= unicode_codepoint_list_get_last_index(priv->codepoint_list))
-        return;
-
     if (!priv->popover) {
         priv->popover = unicode_character_map_zoom_window_new();
         gtk_popover_set_relative_to((GtkPopover *) priv->popover, (GtkWidget *) self);
         GBindingFlags flags = G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE;
         g_object_bind_property(self, "font-desc", priv->popover, "font-desc", flags);
-        g_object_bind_property(self, "active-character", priv->popover, "active-character", flags);
+        g_object_bind_property(self, "active-cell", priv->popover, "active-cell", flags);
     }
 
     gint row = (priv->active_cell - priv->page_first_cell) / priv->cols;
     gint col = unicode_character_map_cell_column(self, priv->active_cell);
 
+    g_autofree gchar *text = get_text_for_cell(self, priv->active_cell);
+    g_object_set(G_OBJECT(priv->popover), "cell-text", text, NULL);
+
     if ((row >= 0 && row < priv->rows) && (col >= 0 && col < priv->cols)) {
         gint x_offset = unicode_character_map_x_offset(self, col);
         gint column_width = unicode_character_map_column_width(self, col);
@@ -655,9 +630,7 @@ unicode_character_map_drag_begin (GtkWidget *widget, GdkDragContext *context)
     GdkMonitor *monitor = gdk_display_get_monitor_at_window(display, window);
     gdk_monitor_get_geometry(monitor, &geometry);
     scale = CLAMP(((0.3 * geometry.height) / (FACTOR_WIDTH * font_size_px)), 1.0, 5.0);
-    drag_surface = create_glyph_surface(charmap,
-                                        unicode_character_map_get_active_character(charmap),
-                                        scale);
+    drag_surface = create_glyph_surface(charmap, scale);
     gtk_drag_set_icon_surface(context, drag_surface);
     cairo_surface_destroy(drag_surface);
     /* no need to chain up */
@@ -665,51 +638,14 @@ unicode_character_map_drag_begin (GtkWidget *widget, GdkDragContext *context)
 }
 
 static void
-unicode_character_map_drag_data_get (G_GNUC_UNUSED GtkWidget *widget,
+unicode_character_map_drag_data_get (GtkWidget *widget,
                                      G_GNUC_UNUSED GdkDragContext *context,
                                      GtkSelectionData *selection_data,
                                      G_GNUC_UNUSED guint info,
                                      G_GNUC_UNUSED guint time)
 {
-    gchar buf[7];
-    gunichar wc = unicode_codepoint_list_get_char(priv->codepoint_list, priv->active_cell);
-    gint n = g_unichar_to_utf8(wc, buf);
-    gtk_selection_data_set_text(selection_data, buf, n);
-    /* no need to chain up */
-    return;
-}
-
-static void
-unicode_character_map_drag_data_received (GtkWidget *widget,
-                                          G_GNUC_UNUSED GdkDragContext *context,
-                                          G_GNUC_UNUSED gint x, G_GNUC_UNUSED gint y,
-                                          GtkSelectionData *selection_data,
-                                          G_GNUC_UNUSED guint info,
-                                          G_GNUC_UNUSED guint time)
-{
-    UnicodeCharacterMap *charmap = UNICODE_CHARACTER_MAP(widget);
-    g_autofree gchar *text = NULL;
-    gunichar wc;
-
-    if (gtk_selection_data_get_length(selection_data) <= 0 ||
-        gtk_selection_data_get_data(selection_data) == NULL)
-        return;
-
-    text = (gchar *) gtk_selection_data_get_text(selection_data);
-
-    if (text == NULL) /* XXX: say something in the statusbar? */
-        return;
-
-    wc = g_utf8_get_char_validated(text, -1);
-
-    if (wc == (gunichar)(-2) || wc == (gunichar)(-1) || wc > UNICODE_UNICHAR_MAX)
-        unicode_character_map_emit_status_message(charmap, _("Unknown character, unable to identify."));
-    else if (unicode_codepoint_list_get_index(priv->codepoint_list, wc) == -1)
-        unicode_character_map_emit_status_message(charmap, _("Not found."));
-    else {
-        unicode_character_map_emit_status_message(charmap, _("Character found."));
-        unicode_character_map_set_active_character(charmap, wc);
-    }
+    g_autofree gchar *text = get_text_for_cell(UNICODE_CHARACTER_MAP(widget), priv->active_cell);
+    gtk_selection_data_set_text(selection_data, text, -1);
     /* no need to chain up */
     return;
 }
@@ -775,13 +711,11 @@ expose_done:
 static gboolean
 unicode_character_map_motion_notify (GtkWidget *widget, GdkEventMotion *event)
 {
-    UnicodeCharacterMap *charmap = UNICODE_CHARACTER_MAP(widget);
     gboolean (* motion_notify_event) (GtkWidget *, GdkEventMotion *) =
     GTK_WIDGET_CLASS (unicode_character_map_parent_class)->motion_notify_event;
 
     if ((event->state & GDK_BUTTON1_MASK) &&
-        gtk_drag_check_threshold(widget, priv->click_x, priv->click_y, event->x, event->y) &&
-        unicode_unichar_validate(unicode_character_map_get_active_character(charmap)))
+        gtk_drag_check_threshold(widget, priv->click_x, priv->click_y, event->x, event->y))
     {
         gtk_drag_begin_with_coordinates(widget,
                                         priv->target_list,
@@ -1043,49 +977,9 @@ unicode_character_map_move_cursor (UnicodeCharacterMap *charmap,
 static void
 unicode_character_map_copy_clipboard (UnicodeCharacterMap *charmap)
 {
-    gunichar wc = unicode_character_map_get_active_character(charmap);
-    if (unicode_unichar_validate(wc)) {
-        gchar utf8[7];
-        gsize len = g_unichar_to_utf8(wc, utf8);
-        GtkClipboard *clipboard = gtk_widget_get_clipboard(GTK_WIDGET(charmap),
-                                                            GDK_SELECTION_CLIPBOARD);
-        gtk_clipboard_set_text(clipboard, utf8, len);
-    }
-    return;
-}
-
-static void
-unicode_character_map_paste_received_cb (G_GNUC_UNUSED GtkClipboard *clipboard,
-                                         const char *text,
-                                         gpointer user_data)
-{
-    gpointer *data = (gpointer *) user_data;
-    UnicodeCharacterMap *charmap = *data;
-
-    g_slice_free(gpointer, data);
-    g_return_if_fail(charmap != NULL);
-    g_object_remove_weak_pointer(G_OBJECT(charmap), data);
-    g_return_if_fail(text != NULL);
-
-    gunichar wc = g_utf8_get_char_validated(text, -1);
-    if (wc == 0 || !unicode_unichar_validate(wc)) {
-        gtk_widget_error_bell(GTK_WIDGET(charmap));
-        return;
-    }
-    unicode_character_map_set_active_character(charmap, wc);
-    return;
-}
-
-static void
-unicode_character_map_paste_clipboard (UnicodeCharacterMap *charmap)
-{
-    g_return_if_fail(gtk_widget_get_realized(GTK_WIDGET(charmap)));
-
-    gpointer *data = g_slice_new(gpointer);
+    g_autofree gchar *text = get_text_for_cell(charmap, priv->active_cell);
     GtkClipboard *clipboard = gtk_widget_get_clipboard(GTK_WIDGET(charmap), GDK_SELECTION_CLIPBOARD);
-    *data = charmap;
-    g_object_add_weak_pointer(G_OBJECT(charmap), data);
-    gtk_clipboard_request_text(clipboard, unicode_character_map_paste_received_cb, data);
+    gtk_clipboard_set_text(clipboard, text, -1);
     return;
 }
 
@@ -1190,8 +1084,8 @@ unicode_character_map_set_property (GObject *gobject,
             priv->vscroll_policy = g_value_get_enum(value);
             gtk_widget_queue_resize(GTK_WIDGET(charmap));
             break;
-        case PROP_ACTIVE_CHAR:
-            unicode_character_map_set_active_character(charmap, g_value_get_uint(value));
+        case PROP_ACTIVE_CELL:
+            priv->active_cell = g_value_get_int(value);
             break;
         case PROP_FONT_DESC:
             unicode_character_map_set_font_desc(charmap, g_value_get_boxed(value));
@@ -1231,8 +1125,8 @@ unicode_character_map_get_property (GObject *gobject,
         case PROP_VSCROLL_POLICY:
             g_value_set_enum(value, priv->vscroll_policy);
             break;
-        case PROP_ACTIVE_CHAR:
-            g_value_set_uint(value, unicode_character_map_get_active_character (charmap));
+        case PROP_ACTIVE_CELL:
+            g_value_set_int(value, priv->active_cell);
             break;
         case PROP_FONT_DESC:
             g_value_set_boxed(value, unicode_character_map_get_font_desc (charmap));
@@ -1264,7 +1158,6 @@ unicode_character_map_class_init (UnicodeCharacterMapClass *klass)
 
     widget_class->drag_begin = unicode_character_map_drag_begin;
     widget_class->drag_data_get = unicode_character_map_drag_data_get;
-    widget_class->drag_data_received = unicode_character_map_drag_data_received;
     widget_class->button_press_event = unicode_character_map_button_press;
     widget_class->get_preferred_width = unicode_character_map_get_preferred_width;
     widget_class->get_preferred_height = unicode_character_map_get_preferred_height;
@@ -1274,10 +1167,9 @@ unicode_character_map_class_init (UnicodeCharacterMapClass *klass)
     widget_class->style_updated = unicode_character_map_style_updated;
 
     klass->activate = NULL;
-    klass->set_active_char = NULL;
+    klass->set_active_cell = NULL;
     klass->move_cursor = unicode_character_map_move_cursor;
     klass->copy_clipboard = unicode_character_map_copy_clipboard;
-    klass->paste_clipboard = unicode_character_map_paste_clipboard;
 
     /* GtkScrollable interface properties */
     g_object_class_override_property(object_class, PROP_HADJUSTMENT, "hadjustment");
@@ -1326,28 +1218,16 @@ unicode_character_map_class_init (UnicodeCharacterMapClass *klass)
                                             G_TYPE_NONE,
                                             0);
 
-    signals[PASTE_CLIPBOARD] = g_signal_new(I_("paste-clipboard"),
-                                            G_OBJECT_CLASS_TYPE(object_class),
-                                            G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
-                                            G_STRUCT_OFFSET(UnicodeCharacterMapClass, paste_clipboard),
-                                            NULL, NULL,
-                                            NULL,//g_cclosure_marshal_VOID__VOID,
-                                            G_TYPE_NONE,
-                                            0);
-
-    /* Not using g_param_spec_unichar on purpose, since it disallows certain
-     * values we want (it's performing a g_unichar_validate).
-     */
     g_object_class_install_property(object_class,
-                                    PROP_ACTIVE_CHAR,
-                                    g_param_spec_uint("active-character",
-                                                        NULL,
-                                                        "Active character",
-                                                        0,
-                                                        UNICODE_UNICHAR_MAX,
-                                                        0,
-                                                        G_PARAM_READWRITE |
-                                                        G_PARAM_STATIC_STRINGS));
+                                    PROP_ACTIVE_CELL,
+                                    g_param_spec_int("active-cell",
+                                                     NULL,
+                                                     "Active cell in character map",
+                                                     G_MININT,
+                                                     G_MAXINT,
+                                                     0,
+                                                     G_PARAM_READWRITE |
+                                                     G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property(object_class,
                                     PROP_FONT_DESC,
@@ -1417,8 +1297,6 @@ unicode_character_map_class_init (UnicodeCharacterMapClass *klass)
     /* Clipboard actions */
     gtk_binding_entry_add_signal(binding_set, GDK_KEY_c, GDK_CONTROL_MASK, "copy-clipboard", 0);
     gtk_binding_entry_add_signal(binding_set, GDK_KEY_Insert, GDK_CONTROL_MASK, "copy-clipboard", 0);
-    gtk_binding_entry_add_signal(binding_set, GDK_KEY_v, GDK_CONTROL_MASK, "paste-clipboard", 0);
-    gtk_binding_entry_add_signal(binding_set, GDK_KEY_Insert, GDK_SHIFT_MASK, "paste-clipboard", 0);
 
     return;
 }
@@ -1454,6 +1332,7 @@ unicode_character_map_set_font_desc (UnicodeCharacterMap *charmap, PangoFontDesc
         return;
 
     unicode_character_map_set_font_desc_internal(charmap, pango_font_description_copy(font_desc));
+    unicode_character_map_emit_status_message(charmap, "font-desc");
     return;
 }
 
@@ -1507,7 +1386,7 @@ unicode_character_map_set_codepoint_list (UnicodeCharacterMap *charmap,
     else
         priv->last_cell = 0;
     g_object_notify(obj, "codepoint-list");
-    g_object_notify(obj, "active-character");
+    g_object_notify(obj, "active-cell");
     gtk_widget_queue_draw(GTK_WIDGET(charmap));
     update_scrollbar_adjustment(charmap);
     g_object_thaw_notify(obj);
@@ -1546,36 +1425,59 @@ unicode_character_map_get_preview_size (UnicodeCharacterMap *charmap)
 }
 
 /**
- * unicode_character_map_get_active_character:
+ * unicode_character_map_get_active_cell:
  * @charmap: a #UnicodeCharacterMap
  *
- * Returns: The currently selected #gunichar
+ * Returns: The currently selected cell
  */
-gunichar
-unicode_character_map_get_active_character (UnicodeCharacterMap *charmap)
+gint
+unicode_character_map_get_active_cell (UnicodeCharacterMap *charmap)
 {
     g_return_val_if_fail(UNICODE_IS_CHARACTER_MAP(charmap), 0);
-    if (!priv->codepoint_list)
-        return 0;
-    return unicode_codepoint_list_get_char(priv->codepoint_list, priv->active_cell);
+    return priv->active_cell;
 }
 
+
 /**
- * unicode_character_map_set_active_character:
+ * unicode_character_map_set_active_cell:
  * @charmap: a #UnicodeCharacterMap
- * @wc: a unicode character (UTF-32)
- *
- * Sets @wc as the selected character.
+ * @cell: cell to select
  */
 void
-unicode_character_map_set_active_character (UnicodeCharacterMap *charmap, gunichar wc)
+unicode_character_map_set_active_cell (UnicodeCharacterMap *charmap, gint cell)
 {
-    g_return_if_fail(UNICODE_IS_CHARACTER_MAP(charmap));
-    gint cell = unicode_codepoint_list_get_index(priv->codepoint_list, wc);
-    if (cell < 0 || cell > priv->last_cell) {
-        gtk_widget_error_bell(GTK_WIDGET(charmap));
+    GtkWidget *widget = GTK_WIDGET(charmap);
+
+    if (cell == priv->active_cell)
         return;
+
+    if (cell < 0)
+        cell = 0;
+    else if (cell > priv->last_cell)
+        cell = priv->last_cell;
+
+    int old_active_cell = priv->active_cell;
+    int old_page_first_cell = priv->page_first_cell;
+
+    priv->active_cell = cell;
+
+    if (cell < priv->page_first_cell || cell >= priv->page_first_cell + priv->page_size) {
+        int old_row = old_active_cell / priv->cols;
+        int new_row = cell / priv->cols;
+        int new_page_first_cell = old_page_first_cell + ((new_row - old_row) * priv->cols);
+        int last_row = (priv->last_cell / priv->cols) + 1;
+        int last_page_first_row = last_row - priv->rows;
+        int last_page_first_cell = (last_page_first_row * priv->cols) + 1;
+        priv->page_first_cell = CLAMP(new_page_first_cell, 0, last_page_first_cell);
+        if (priv->vadjustment)
+            gtk_adjustment_set_value(priv->vadjustment, priv->page_first_cell / priv->cols);
+    } else if (gtk_widget_get_realized(widget)) {
+        /* Clear previous selection */
+        expose_cell(charmap, old_active_cell);
+        /* Update selected cell */
+        expose_cell(charmap, cell);
     }
-    unicode_character_map_set_active_cell(charmap, cell);
+
+    g_object_notify(G_OBJECT(charmap), "active-cell");
     return;
 }
