@@ -2,7 +2,7 @@
  *
  * Originally a part of Gucharmap
  *
- * Copyright (C) 2017 - 2021 Jerry Casiano
+ * Copyright (C) 2017-2022 Jerry Casiano
  *
  *
  * Copyright © 2004 Noah Levitt
@@ -48,16 +48,16 @@ struct _UnicodeCodepointListInterface
     /*<private>*/
     GTypeInterface parent_iface;
     /*<public>*/
-    gunichar  (* get_char)         (UnicodeCodepointList *self, gint index);
+    GSList *    (* get_codepoints)  (UnicodeCodepointList *self, gint index);
     /* zero is the first index */
-    gint      (* get_index)        (UnicodeCodepointList *self, gunichar wc);
-    gint      (* get_last_index)   (UnicodeCodepointList *self);
+    gint        (* get_index)       (UnicodeCodepointList *self, GSList *codepoints);
+    gint        (* get_last_index)  (UnicodeCodepointList *self);
 
 };
 
-gunichar    unicode_codepoint_list_get_char       (UnicodeCodepointList *self, gint index);
-gint        unicode_codepoint_list_get_index      (UnicodeCodepointList *self, gunichar wc);
-gint        unicode_codepoint_list_get_last_index (UnicodeCodepointList *self);
+GSList *    unicode_codepoint_list_get_codepoints   (UnicodeCodepointList *self, gint index);
+gint        unicode_codepoint_list_get_index        (UnicodeCodepointList *self, GSList *codepoints);
+gint        unicode_codepoint_list_get_last_index   (UnicodeCodepointList *self);
 
 G_END_DECLS
 
