@@ -2,7 +2,7 @@
  *
  * Originally a part of Gucharmap
  *
- * Copyright (C) 2017 - 2021 Jerry Casiano
+ * Copyright (C) 2017-2022 Jerry Casiano
  *
  *
  * Copyright © 2004 Noah Levitt
@@ -28,6 +28,9 @@
 #define __UNICODE_INFO_H__
 
 #include <glib.h>
+#include <glib/gi18n-lib.h>
+
+#include "unicode-codepoint-list.h" /* UNICODE_UNICHAR_MAX */
 
 G_BEGIN_DECLS
 
@@ -94,27 +97,16 @@ gint unicode_get_codepoint_data_name_count (void);
 gint unicode_get_unihan_count (void);
 gint unicode_unichar_to_printable_utf8 (gunichar uc, gchar *outbuf);
 gboolean unicode_unichar_validate (gunichar  uc);
-gboolean unicode_unichar_isdefined (gunichar  uc);
 gboolean unicode_unichar_isgraph (gunichar  uc);
 gunichar * unicode_get_nameslist_exes (gunichar  uc);
 const gchar * unicode_get_codepoint_name (gunichar uc);
 const gchar * unicode_get_codepoint_data_name (gunichar uc);
 const gchar * unicode_get_category_name (gunichar uc);
 const gchar * unicode_get_unicode_kDefinition (gunichar uc);
-const gchar * unicode_get_unicode_kCantonese (gunichar uc);
-const gchar * unicode_get_unicode_kMandarin (gunichar uc);
-const gchar * unicode_get_unicode_kTang (gunichar uc);
-const gchar * unicode_get_unicode_kKorean (gunichar uc);
-const gchar * unicode_get_unicode_kJapaneseKun (gunichar uc);
-const gchar * unicode_get_unicode_kJapaneseOn (gunichar uc);
-const gchar * unicode_get_unicode_kHangul (gunichar uc);
-const gchar * unicode_get_unicode_kVietnamese (gunichar uc);
-const gchar *  unicode_get_script_for_char (gunichar wc);
 const gchar ** unicode_get_nameslist_stars (gunichar  uc);
 const gchar ** unicode_get_nameslist_equals (gunichar  uc);
 const gchar ** unicode_get_nameslist_pounds (gunichar  uc);
 const gchar ** unicode_get_nameslist_colons (gunichar  uc);
-const gchar ** unicode_list_scripts (void);
 const gchar * unicode_version_to_string (UnicodeStandard version);
 UnicodeStandard unicode_get_version (gunichar uc);
 
