@@ -2,7 +2,7 @@
  *
  * Originally a part of Gucharmap
  *
- * Copyright (C) 2017 - 2021 Jerry Casiano
+ * Copyright (C) 2017-2022 Jerry Casiano
  *
  *
  * Copyright © 2004 Noah Levitt
@@ -203,15 +203,23 @@ check_for_explicit_codepoint (const UnicodeCodepointList *codepoint_list, const
 
     if (nptr != string) {
         wc = strtoul(nptr, &endptr, base);
-        if (endptr != nptr)
-            index = unicode_codepoint_list_get_index((UnicodeCodepointList *) codepoint_list, wc);
+        if (endptr != nptr) {
+            GSList *codepoints = NULL;
+            codepoints = g_slist_append(codepoints, GINT_TO_POINTER(wc));
+            index = unicode_codepoint_list_get_index((UnicodeCodepointList *) codepoint_list, codepoints);
+            g_slist_free(codepoints);
+        }
     }
 
     /* Check for hex codepoint without any prefix. */
     if (index < 0 && base > 10) {
         wc = strtoul(string, &endptr, base);
-        if (endptr-3 >= string)
-            index = unicode_codepoint_list_get_index((UnicodeCodepointList *) codepoint_list, wc);
+        if (endptr-3 >= string) {
+            GSList *codepoints = NULL;
+            codepoints = g_slist_append(codepoints, GINT_TO_POINTER(wc));
+            index = unicode_codepoint_list_get_index((UnicodeCodepointList *) codepoint_list, codepoints);
+            g_slist_free(codepoints);
+        }
     }
 
     return index;
@@ -258,7 +266,10 @@ quick_checks_after (UnicodeSearchState *search_state)
 {
     /* jump to the first nonspace character unless it’s plain ascii */
     if (search_state->search_string_nfd[0] < 0x20 || search_state->search_string_nfd[0] > 0x7e) {
-        gint index = unicode_codepoint_list_get_index (search_state->codepoint_list, g_utf8_get_char (search_state->search_string_nfd));
+        GSList *codepoints = NULL;
+        codepoints = g_slist_append(codepoints, GINT_TO_POINTER(g_utf8_get_char(search_state->search_string_nfd)));
+        gint index = unicode_codepoint_list_get_index (search_state->codepoint_list, codepoints);
+        g_slist_free(codepoints);
         if (index != -1) {
             search_state->match = index;
             search_state->search_complete = TRUE;
@@ -281,14 +292,10 @@ idle_search (UnicodeSearchBar *self)
 
     timer = g_timer_new ();
 
-    gint n_chars = unicode_codepoint_list_get_last_index(self->search_state->codepoint_list) + 1;
+    gint n_chars = unicode_codepoint_list_get_last_index(self->search_state->codepoint_list);
 
     do {
         self->search_state->curr_index = (self->search_state->curr_index + self->search_state->direction + n_chars) % n_chars;
-        wc = unicode_codepoint_list_get_char (self->search_state->codepoint_list, self->search_state->curr_index);
-
-        if (!unicode_unichar_validate (wc) || !unicode_unichar_isdefined (wc))
-            continue;
 
         /* check for explicit codepoint */
         if (self->search_state->search_string_value != -1 && self->search_state->curr_index == self->search_state->search_string_value) {
@@ -298,13 +305,26 @@ idle_search (UnicodeSearchBar *self)
             return FALSE;
         }
 
-        /* check for other matches */
-        if (matches(wc, self->search_state->search_string_nfd)) {
-            self->search_state->match = self->search_state->curr_index;
-            g_timer_destroy (timer);
-            return FALSE;
+        GSList *codepoints = unicode_codepoint_list_get_codepoints(self->search_state->codepoint_list, self->search_state->curr_index);
+
+        for (GSList *iter = codepoints; iter != NULL; iter = iter->next) {
+
+            wc = (gunichar) GPOINTER_TO_INT(iter->data);
+
+            if (!unicode_unichar_validate (wc))
+                continue;
+
+            /* check for other matches */
+            if (matches(wc, self->search_state->search_string_nfd)) {
+                self->search_state->match = self->search_state->curr_index;
+                g_timer_destroy (timer);
+                return FALSE;
+            }
+
         }
 
+        g_slist_free(codepoints);
+
         if (g_timer_elapsed (timer, NULL) > 0.050) {
             g_timer_destroy (timer);
             return TRUE;
@@ -369,18 +389,24 @@ unicode_search_state_new (UnicodeCodepointList *codepoint_list,
     search_state->search_string_nfd = g_utf8_normalize(search_string, -1, G_NORMALIZE_NFD);
     search_state->search_string_nfd_len = g_utf8_strlen(search_state->search_string_nfd, -1);
 
-    if (search_state->search_string_nfd_len == 1)
-        search_state->search_index_nfd = unicode_codepoint_list_get_index(search_state->codepoint_list, g_utf8_get_char(search_state->search_string_nfd));
-    else
+    if (search_state->search_string_nfd_len == 1) {
+        GSList *codepoints = NULL;
+        codepoints = g_slist_append(codepoints, GINT_TO_POINTER(g_utf8_get_char(search_state->search_string_nfd)));
+        search_state->search_index_nfd  = unicode_codepoint_list_get_index (search_state->codepoint_list, codepoints);
+        g_slist_free(codepoints);
+    } else
         search_state->search_index_nfd = -1;
 
     /* NFC */
     search_state->search_string_nfc = g_utf8_normalize(search_state->search_string_nfd, -1, G_NORMALIZE_NFC);
     search_state->search_string_nfc_len = g_utf8_strlen (search_state->search_string_nfc, -1);
 
-    if (search_state->search_string_nfc_len == 1)
-        search_state->search_index_nfc = unicode_codepoint_list_get_index(search_state->codepoint_list, g_utf8_get_char(search_state->search_string_nfc));
-    else
+    if (search_state->search_string_nfc_len == 1) {
+        GSList *codepoints = NULL;
+        codepoints = g_slist_append(codepoints, GINT_TO_POINTER(g_utf8_get_char(search_state->search_string_nfc)));
+        search_state->search_index_nfc  = unicode_codepoint_list_get_index(search_state->codepoint_list, codepoints);
+        g_slist_free(codepoints);
+    } else
         search_state->search_index_nfc = -1;
 
     /* INDEX */
@@ -403,13 +429,9 @@ search_completed (UnicodeSearchBar *self)
 {
     g_return_if_fail(self != NULL && self->charmap != NULL);
     UnicodeSearchState *search_state = self->search_state;
-    gunichar found_char = -1;
-
-    if (search_state->match >= 0)
-        found_char = unicode_codepoint_list_get_char(search_state->codepoint_list, search_state->match);
-
+    gint index = search_state->match >= 0 ? search_state->match : -1;
     search_state->searching = FALSE;
-    unicode_character_map_set_active_character(self->charmap, found_char);
+    unicode_character_map_set_active_cell(self->charmap, index);
     set_action_visibility(self, !search_state->search_complete);
     return;
 }
@@ -420,7 +442,6 @@ unicode_search_start (UnicodeSearchBar *self, UnicodeSearchDirection direction)
     g_return_if_fail(self != NULL && self->charmap != NULL);
 
     gint start_index;
-    gunichar start_char;
     g_autoptr(UnicodeCodepointList) codepoint_list = NULL;
 
     if (self->search_state && self->search_state->searching) /* Already searching */
@@ -436,14 +457,12 @@ unicode_search_start (UnicodeSearchBar *self, UnicodeSearchDirection direction)
 
         g_clear_pointer(&self->search_state, unicode_search_state_free);
 
-        start_char = unicode_character_map_get_active_character(self->charmap);
-        start_index = unicode_codepoint_list_get_index(codepoint_list, start_char);
+        start_index = unicode_character_map_get_active_cell(self->charmap);
         self->search_state = unicode_search_state_new(codepoint_list,
                                                       gtk_entry_get_text(GTK_ENTRY(self->entry)),
                                                       start_index, direction );
     } else {
-        start_char = unicode_character_map_get_active_character (self->charmap);
-        self->search_state->start_index = unicode_codepoint_list_get_index (codepoint_list, start_char);
+        self->search_state->start_index = unicode_character_map_get_active_cell(self->charmap);
         self->search_state->curr_index = self->search_state->start_index;
         self->search_state->direction = direction;
     }
@@ -498,9 +517,7 @@ entry_changed (UnicodeSearchBar *self, G_GNUC_UNUSED GtkWidget *widget)
         }
         search_timeout = g_timeout_add(500, (GSourceFunc) _entry_changed, self);
     } else {
-        UnicodeCodepointList *codepoint_list = unicode_character_map_get_codepoint_list(self->charmap);
-        gunichar first_char = unicode_codepoint_list_get_char(codepoint_list, 0);
-        unicode_character_map_set_active_character(self->charmap, first_char);
+        unicode_character_map_set_active_cell(self->charmap, 0);
     }
 
     return;
@@ -611,6 +628,20 @@ unicode_search_bar_class_init (UnicodeSearchBarClass *klass)
     return;
 }
 
+static void
+reset_search (G_GNUC_UNUSED UnicodeCharacterMap *charmap,
+              G_GNUC_UNUSED const gchar *status_message,
+              UnicodeSearchBar *self)
+{
+    g_return_if_fail(self != NULL);
+    if (gtk_widget_get_mapped(GTK_WIDGET(self->entry)) && gtk_entry_get_text_length(self->entry) > 0) {
+        g_autofree gchar *previous_text = g_strdup(gtk_entry_get_text(self->entry));
+        gtk_entry_set_text(self->entry, " ");
+        gtk_entry_set_text(self->entry, previous_text);
+    }
+    return;
+}
+
 /**
  * unicode_search_bar_set_character_map:
  * @self:                                       #UnicodeSearchBar
@@ -621,8 +652,12 @@ unicode_search_bar_set_character_map (UnicodeSearchBar *self,
                                       UnicodeCharacterMap *character_map)
 {
     g_return_if_fail(self != NULL);
+    if (self->charmap)
+        g_signal_handlers_disconnect_by_func(self->charmap, G_CALLBACK(reset_search), self);
     if (g_set_object(&self->charmap, character_map))
         g_object_notify_by_pspec(G_OBJECT(self), obj_properties[PROP_CHARMAP]);
+    if (self->charmap)
+        g_signal_connect_after(self->charmap, "status-message", G_CALLBACK(reset_search), self);
     return;
 }
 
